package de.fhdw.wtf.generator.transformer.visitorTransformation;

import java.util.Iterator;

import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassModifierService;
import de.fhdw.wtf.common.ast.type.ClassModifierTransient;
import de.fhdw.wtf.common.ast.type.ClassModifierVisitable;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.visitor.ClassModifierVisitorReturn;

public final class UtilVisitorTransformer {
	
	public static final String HANDLE_OPNAME = "handle";
	public static final String ACCEPT_OPNAME = "accept";
	public static final String ACCEPT_METHOD_WITHOUT_RETURN = "visitor.handle(this);";
	public static final String ACCEPT_METHOD_RETURN = "return " + ACCEPT_METHOD_WITHOUT_RETURN;
	public static final String ACCEPT_PARAMETER_NAME = "visitor";
	public static final String FULL_VISITOR_SUFFIX = "$Visitor";
	public static final String PARTIAL_VISITOR_SUFFIX = "$PartialVisitor";
	public static final String DEFAULT_VISITOR_COMMENT = "This class represents a visitor.";
	public static final String DEFAULT_HANDLE_COMMENT = "This operation is part of a visitor.";
	public static final String DEFAULT_ACCEPT_COMMENT = "This is an accept-Operation for a visitor.";
	public static final String DEFAULT_HANDLE_METHOD = "// TODO: Implement method!";
	public static final char JAVA_LINE_END = ';';
	public static final String RETURN_VISITOR = "Return";
	public static final String EXCEPTION_VISITOR = "Exception";
	
	private UtilVisitorTransformer() {
	}
	
	public static String firstToLowerCase(final String string) {
		final String first = string.substring(0, 1);
		final String firstLower = first.toLowerCase();
		return firstLower + string.substring(1);
	}
	
	/**
	 * Returns true if <code>c</code> has the {@link ClassModifier} {@link ClassModifierVisitable}. Else returns false.
	 * 
	 * @param c
	 * @return
	 */
	public static boolean isVisitable(final ClassType c) {
		final Iterator<ClassModifier> i = c.getModifiers().iterator();
		boolean result = false;
		while (!result && i.hasNext()) {
			final ClassModifier current = i.next();
			result = current.accept(new ClassModifierVisitorReturn<Boolean>() {
				@Override
				public Boolean handle(final ClassModifierAbstract modAbstract) {
					return false;
				}
				
				@Override
				public Boolean handle(final ClassModifierService modService) {
					return false;
				}
				
				@Override
				public Boolean handle(final ClassModifierTransient modTransient) {
					return false;
				}
				
				@Override
				public Boolean handle(final ClassModifierVisitable visitable) {
					return true;
				}
			});
		}
		return result;
	}
}
